/*
	Global functions for the AIS Dependability Testbench

	2005. 11. 14.	Zoltan Micskei	created
	2006. 01. 20.	Zoltan Micskei	refactored all functions to return char*
	2006. 11. 13.	Zoltan Micskei	removed functions for types not in AIS.B.01	
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <errno.h>

#include "saAis.h"
#include "saAmf.h"
#include "saClm.h"

#include "testCommon.h"

extern int errno;

/* ------------------------- SaNameT --------------------------------- */

char* SaNameToString (SaNameT *name)
{    	
	if (name == NULL)
	{
        return "NULL";
	}
	else
	{
        return (char*) name->value;
	}	
}

void setSaNameT (SaNameT *name, char *str) 
{
	name->length = strlen (str);
	memcpy (name->value, str, name->length);
}

/* ------------------------- SaDispatchFlagsT --------------------------------- */
char* SaDispatchFlagsToString(SaDispatchFlagsT* flag)
{
	if (flag == NULL)
	{
		return "NULL";
	}
	else
	{
		switch(*flag)
		{
			case SA_DISPATCH_ONE:
				return "SA_DISPATCH_ONE";
			case SA_DISPATCH_ALL:
				return "SA_DISPATCH_ALL";
			case SA_DISPATCH_BLOCKING:
				return "SA_DISPATCH_BLOCKING";
			default:
				return "invalid_SaDispatchFlagsT";
		}
    }
}

/* ------------------------- SaVersionT --------------------------------- */

char* SaVersionToString(SaVersionT* version)
{
    static char buffer[70];
    
    if (version == NULL)
    {
        return "NULL";
    }
    else
    {
        sprintf(buffer, "%c.%u.%u (%p)", version->releaseCode, version->majorVersion, version->minorVersion, version);
    }
    
    return buffer;
}

/* ------------------------- SaAisErrorT --------------------------------- */
char* SaAisErrorToString(SaAisErrorT* error)
{
	if ( error == NULL )
	{
		return "NULL";
	}
    else
    {
		switch(*error) {
			case SA_AIS_OK:
				return "SA_AIS_OK";
			case SA_AIS_ERR_LIBRARY:
				return "SA_AIS_ERR_LIBRARY";
			case SA_AIS_ERR_VERSION:
				return "SA_AIS_ERR_VERSION";
			case SA_AIS_ERR_INIT:
				return "SA_AIS_ERR_INIT";
			case SA_AIS_ERR_TIMEOUT:
				return "SA_AIS_ERR_TIMEOUT";
			case SA_AIS_ERR_TRY_AGAIN:
				return "SA_AIS_ERR_TRY_AGAIN";
			case SA_AIS_ERR_INVALID_PARAM:
				return "SA_AIS_ERR_INVALID_PARAM";
			case SA_AIS_ERR_NO_MEMORY:
				return "SA_AIS_ERR_NO_MEMORY";
			case SA_AIS_ERR_BAD_HANDLE:
				return "SA_AIS_ERR_BAD_HANDLE";
			case SA_AIS_ERR_BUSY:
				return "SA_AIS_ERR_BUSY";
			case SA_AIS_ERR_ACCESS:
				return "SA_AIS_ERR_ACCESS";
			case SA_AIS_ERR_NOT_EXIST:
				return "SA_AIS_ERR_NOT_EXIST";
			case SA_AIS_ERR_NAME_TOO_LONG:
				return "SA_AIS_ERR_NAME_TOO_LONG";
			case SA_AIS_ERR_EXIST:
				return "SA_AIS_ERR_EXIST";
			case SA_AIS_ERR_NO_SPACE:
				return "SA_AIS_ERR_NO_SPACE";
			case SA_AIS_ERR_INTERRUPT:
				return "SA_AIS_ERR_INTERRUPT";
			case SA_AIS_ERR_NAME_NOT_FOUND:
				return "SA_AIS_ERR_NAME_NOT_FOUND";
			case SA_AIS_ERR_NO_RESOURCES:
				return "SA_AIS_ERR_NO_RESOURCES";
			case SA_AIS_ERR_NOT_SUPPORTED:
				return "SA_AIS_ERR_NOT_SUPPORTED";
			case SA_AIS_ERR_BAD_OPERATION:
				return "SA_AIS_ERR_BAD_OPEARATION";
			case SA_AIS_ERR_FAILED_OPERATION:
				return "SA_AIS_ERR_FAILED_OPERATION";
			case SA_AIS_ERR_MESSAGE_ERROR:
				return "SA_AIS_ERR_MESSAGE_ERROR";
			case SA_AIS_ERR_QUEUE_FULL:
				return "SA_AIS_ERR_QUEUE_FULL";
			case SA_AIS_ERR_QUEUE_NOT_AVAILABLE:
				return "SA_AIS_ERR_QUEUE_NOT_AVAILABLE";
			case SA_AIS_ERR_BAD_FLAGS:
				return "SA_AIS_ERR_BAD_FLAGS";
			case SA_AIS_ERR_TOO_BIG:
				return "SA_AIS_ERR_TOO_BIG";
			case SA_AIS_ERR_NO_SECTIONS:
				return "SA_AIS_ERR_NO_SECTIONS";
			default:
				return "(invalid error code)";
		} // switch
	} // if
}

/* ------------------------- SaSelectionObjectT --------------------------------- */
char* SaSelectionObjectToString(SaSelectionObjectT* selectionObject)
{
    static char buffer[100];

	if ( selectionObject == NULL )
	{
		return "NULL";
	}
	else
	{
		sprintf( buffer, "%llu (%p)", *selectionObject, selectionObject );
	}
	
	return buffer;
}

/* ------------------------- SaInvocationT --------------------------------- */
char* SaInvocationToString(SaInvocationT* invocation)
{
	static char buffer[100];

	if ( invocation == NULL )
	{
		return "NULL";
	}
	else
	{
		sprintf( buffer, "%llu (%p)", *invocation, invocation );
		return buffer;
	}
	
	
}

/* ------------------------- SaTimeT --------------------------------- */
char* SaTimeToString(SaTimeT* t)
{
	static char buffer[100];

	if ( t == NULL )
	{
		return "NULL";
	}
	else
	{
		sprintf( buffer, "%lld (%p)", *t, t );
		return buffer;
	}
}

/* ------------------------- SaUint8T --------------------------------- */
char* SaUint8ToString(SaUint8T* i)
{
	static char buffer[100];

	if ( i == NULL )
	{
		return "NULL";
	}
	else
	{
		sprintf( buffer, "%u (%p)", *i, i );
		return buffer;
	}
}

/* ------------------------- SaInt32T --------------------------------- */
char* SaInt32ToString(SaInt32T *i)
{
	static char buffer[100];

	if ( i == NULL )
	{
		return "NULL";
	}
	else
	{
		sprintf( buffer, "%d (%p)", *i, i );
		return buffer;
	}
}

/* ------------------------- SaUint32T --------------------------------- */
char* SaUint32ToString(SaUint32T* i)
{
	static char buffer[100];

	if ( i == NULL )
	{
		return "NULL";
	}
	else
	{
		sprintf( buffer, "%u (%p)", *i, i );
		return buffer;
	}
}

/* ------------------------- SaUint64T --------------------------------- */
char* SaUint64ToString(SaUint64T* i)
{
	static char buffer[100];

	if ( i == NULL )
	{
		return "NULL";
	}
	else
	{
		sprintf( buffer, "%llu (%p)", *i, i );
		return buffer;
	}
}

/* ------------------------- SaNtfIdentifierT --------------------------------- */
char* SaNtfIdentifierToString(SaNtfIdentifierT* identifier)
{
	static char buffer[100];
	
	if ( identifier == NULL )
	{
		return "NULL";
	}
	else
	{
		sprintf( buffer, "%llu (%p)", *identifier, identifier );
		return buffer;
	}	
}

/* ----------------------------- AMF -------------------------------------- */
/* ------------------------- SaAmfHandleT --------------------------------- */
char* SaAmfHandleToString (SaAmfHandleT* handle)
{
    static char buffer[100];

	if ( handle == NULL )
	{
		return "NULL";
	}
	else
	{
		sprintf( buffer, "%llu (%p)", *handle, handle );
	}
	
	return buffer;
}

/* ------------------------- SaAmfCallbacksT --------------------------------- */
char* SaAmfCallbacksToString(SaAmfCallbacksT* callBacks)
{
    static char buffer[100];

    if ( callBacks == NULL )
    {
        return "NULL";
    }
    else
    {
        sprintf(buffer, "AmfCallbacksT (%p)", callBacks);
    }
    
    return buffer;
}

/* ------------------------- SaAmfHAStateT --------------------------------- */
char* SaAmfHAStateToString(SaAmfHAStateT* haState)
{
	static char buffer[80];

	if ( haState == NULL )
	{
		return "NULL";
	}
	else
	{
		switch ( *haState )
		{
			case SA_AMF_HA_ACTIVE:
				return "SA_AMF_HA_ACTIVE";
			case SA_AMF_HA_STANDBY:
				return "SA_AMF_HA_STANDBY";
			case SA_AMF_HA_QUIESCED:
				return "SA_AMF_HA_QUIESCED";
			case SA_AMF_HA_QUIESCING:
				return "SA_AMF_HA_QUIESCING";
			default:
				sprintf(buffer, "invalid SaAmfHAStateT (%d)", *haState);
				return buffer;
		}
	}
}

/* ------------------------- SaAmfProtectionGroupChangesT ------------------------------ */
char* SaAmfProtectionGroupChangesToString(SaAmfProtectionGroupChangesT* change)
{
	static char buffer[80];

	if ( change == NULL )
	{
		return "NULL";
	}
	else
	{
		switch (*change)
		{
			case SA_AMF_PROTECTION_GROUP_NO_CHANGE:
				return "SA_AMF_PROTECTION_GROUP_NO_CHANGE";
			case SA_AMF_PROTECTION_GROUP_ADDED:
				return "SA_AMF_PROTECTION_GROUP_ADDED";
			case SA_AMF_PROTECTION_GROUP_REMOVED:
				return "SA_AMF_PROTECTION_GROUP_REMOVED";
			case SA_AMF_PROTECTION_GROUP_STATE_CHANGE:
				return "SA_AMF_PROTECTION_GROUP_STATE_CHANGE";
			default:
				sprintf(buffer, "invalid SaAmfProtectionGroupChangesT (%d)", *change );
				return buffer;
		}			
	}
}

/* ------------------------- SaAmfProtectionGroupMemberT ------------------------------ */
char* SaAmfProtectionGroupMemberToString(SaAmfProtectionGroupMemberT* pgMember)
{
	static char buffer[100]; //TODO is this length enough?

    if ( pgMember == NULL )
	{
		return "NULL";
	}
	else
	{
		sprintf( buffer, "[%s|%s|%u]", SaNameToString( &(pgMember->compName) ), SaAmfHAStateToString( &(pgMember->haState) ), pgMember->rank );        
		return buffer;
	}
}

/* ------------------------- SaAmfProtectionGroupNotificationT ------------------------------ */
char* SaAmfProtectionGroupNotificationToString(SaAmfProtectionGroupNotificationT* notification)
{
    static char buffer[180];

    if ( notification == NULL )
	{
		return "NULL";
	}
	else
	{
		sprintf( buffer, "[%s|%s]", SaAmfProtectionGroupMemberToString( &(notification->member) ), SaAmfProtectionGroupChangesToString( &(notification->change) ) );        
		return buffer;
	}
}

//TODO in some versions of openais it is mispelled: SaAmfProtectionGruopNotificationBufferT
/* ------------------------- SaAmfProtectionGroupNotificationBufferT ------------------------------ */
char* SaAmfProtectionGroupNotificationBufferToString(SaAmfProtectionGroupNotificationBufferT* notificationBuffer)
{
	static char buffer[180];

    if ( notificationBuffer == NULL )
	{
		return "NULL";
	}
	else
	{
		//TODO write out also. but it can produce segfault easily
		sprintf( buffer, "[notification|%u]", notificationBuffer->numberOfItems );        
		return buffer;
	}
}

/* ------------------------- SaAmfRecommendedRecoveryT ------------------------------ */
char* SaAmfRecommendedRecoveryToString(SaAmfRecommendedRecoveryT* recovery)
{
	static char buffer[80];

	if ( recovery == NULL )
	{
		return "NULL";
	}
	else
	{
		switch (*recovery)
		{
			case SA_AMF_NO_RECOMMENDATION:
				return "SA_AMF_NO_RECOMMENDATION";
			case SA_AMF_COMPONENT_RESTART:
				return "SA_AMF_COMPONENT_RESTART";
			case SA_AMF_COMPONENT_FAILOVER:
				return "SA_AMF_COMPONENT_FAILOVER";
			case SA_AMF_NODE_SWITCHOVER:
				return "SA_AMF_NODE_SWITCHOVER";
			case SA_AMF_NODE_FAILOVER:
				return "SA_AMF_NODE_FAILOVER";
			case SA_AMF_NODE_FAILFAST:
				return "SA_AMF_NODE_FAILFAST";
			case SA_AMF_CLUSTER_RESET:
				return "SA_AMF_CLUSTER_RESET";
			default:
				sprintf(buffer, "invalid SaAmfRecommendedRecoveryT (%d)", *recovery );
				return buffer;
		}			
	}
}

/* ------------------------- SaAmfHealthcheckKeyT --------------------------------- */
char* SaAmfHealthcheckKeyToString(SaAmfHealthcheckKeyT* keys)
{
	static char buffer[80];

	if ( keys == NULL )
	{
		return "NULL";
	}
	else
	{
		sprintf( buffer, "[key|%u]", keys->keyLen );
		return buffer; 	
	}
}

/* ------------------------- SaAmfHealthcheckInvocationT --------------------------------- */
char* SaAmfHealthcheckInvocationToString(SaAmfHealthcheckInvocationT* invocation)
{
	static char buffer[80];

	if ( invocation == NULL )
	{
		return "NULL";
	}
	else
	{
		switch (*invocation)
		{
			case SA_AMF_HEALTHCHECK_AMF_INVOKED:
				return "SA_AMF_HEALTHCHECK_AMF_INVOKED";
			case SA_AMF_HEALTHCHECK_COMPONENT_INVOKED:
				return "SA_AMF_HEALTHCHECK_COMPONENT_INVOKED";
			default:
				sprintf(buffer, "invalid SaAmfHealthcheckInvocationT (%d)", *invocation );
				return buffer;
		}			
	}
}

/* ------------------------- SaAmfPmErrorsT --------------------------------- */
char* SaAmfPmErrorsToString(SaAmfPmErrorsT* errors)
{
	static char buffer[80];

	if ( errors == NULL )
	{
		return "NULL";
	}
	else
	{
		switch (*errors)
		{
			case SA_AMF_PM_ZERO_EXIT:
				return "SA_AMF_PM_ZERO_EXIT";
			case SA_AMF_PM_NON_ZERO_EXIT:
				return "SA_AMF_PM_NON_ZERO_EXIT";				
			case SA_AMF_PM_ABNORMAL_END:
				return "SA_AMF_PM_ABNORMAL_END";
			default:
				sprintf(buffer, "invalid SaAmfPmErrorsT (%u)", *errors );
				return buffer;
		}			
	}
}

/* ------------------------- SaAmfPmStopQualifierT ------------------------ */
char* SaAmfPmStopQualifierToString(SaAmfPmStopQualifierT* qualifier)
{
	static char buffer[80];

	if ( qualifier == NULL )
	{
		return "NULL";
	}
	else
	{
		switch (*qualifier)
		{
			case SA_AMF_PM_PROC:
				return "SA_AMF_PM_PROC";
			case SA_AMF_PM_PROC_AND_DESCENDENTS:			
				return "SA_AMF_PM_PROC_AND_DESCENDENTS";				
			case SA_AMF_PM_ALL_PROCESSES:
				return "SA_AMF_PM_ALL_PROCESSES";
			default:
				sprintf(buffer, "invalid SaAmfPmStopQualifierT (%u)", *qualifier );
				return buffer;
		}			
	}	
}

/* ----------------------------- CLM -------------------------------------- */
/* ------------------------- SaClmHandleT --------------------------------- */
char* SaClmHandleToString(SaClmHandleT* handle)
{
    static char buffer[100];

    if ( handle == NULL )
	{
		return "NULL";
	}
	else
	{
		sprintf( buffer, "%llu (%p)", *handle, handle );        
		return buffer;
	}		
}

/* ------------------------- SaClmCallbacksT --------------------------------- */
char* SaClmCallbacksToString(SaClmCallbacksT* callBacks)
{
    static char buffer[100];

    if ( callBacks == NULL )
    {
        return "NULL";
    }
    else
    {
        sprintf(buffer, "ClmCallbacksT (%p)", callBacks);
    }
    
    return buffer;
}

/* ------------------------- SaClmClusterNodeT --------------------------------- */
char* SaClmClusterNodeToString(SaClmClusterNodeT* node)
{
    static char buffer[255];

    if ( node == NULL )
    {
        return "NULL";
    }
    else
    {
        sprintf(buffer, "[%u|address|%s|%d|%s|%llu]", node->nodeId, SaNameToString(&(node->nodeName)), node->member, SaTimeToString(&(node->bootTimestamp)), node->initialViewNumber);
    }
    
    return buffer;
}



/* --------------------- SaClmClusterNotificationBufferT --------------------------------- */
char* SaClmClusterNotificationBufferToString(SaClmClusterNotificationBufferT* notificationBuffer)
{
    static char buffer[100];

    if ( notificationBuffer == NULL )
    {
        return "NULL";
    }
    else
    {
        sprintf(buffer, "[%llu|%u|(%p)]", notificationBuffer->viewNumber, notificationBuffer->numberOfItems, notificationBuffer->notification);
    }
    
    return buffer;
}

/* ------------------------- SaClmNodeIdT --------------------------------- */
char* SaClmNodeIdToString(SaClmNodeIdT* nodeId)
{
    static char buffer[100];

    if ( nodeId == NULL )
    {
        return "NULL";
    }
    else
    {
        sprintf(buffer, "%u", *nodeId);
    }
    
    return buffer;
}

/* ------------------------- Testcase logging --------------------------------- */
/* Creates a name for the logfile from the function name and the current date */
void createFileName(char * fileName)
{
    int i; char temp[50];

    time_t rawtime; struct tm * timeStruct;
    time ( &rawtime ); timeStruct = localtime( &rawtime );
        
    sprintf( temp, "_%d.%d.%d_%d.%d.%d", (timeStruct->tm_year) + 1900, (timeStruct->tm_mon) + 1, timeStruct->tm_mday, timeStruct->tm_hour, timeStruct->tm_min, timeStruct->tm_sec );
    strcat( fileName, temp );
	strcat( fileName, ".csv" );
	
	// replace : with .
	while( (i = strcspn(fileName, ":")) != strlen(fileName) )
	{
        fileName[i] = '.';
	}
}

void openLogFile(FILE** logFile, char* typeName, SaBoolT writeHeader)
{
	char fileName[250] = LOG_FILES_PREFIX;
	strcat(fileName, typeName);
	
	createFileName(fileName);
	*logFile = fopen( fileName, "w" );	
	if ( (*logFile) == NULL )
	{
		int errorNumber = errno;
		printf("ERROR>Could not open logFile, errno: %s\n", strerror(errorNumber) );
		exit(1);
	}	
}
